/*----------------------------------------------------------------------------------*/
/*	Function:  Sin HID & COM Series Interface Function								*/
/* 	Version:   V1.11												   				*/
/*	Last Date: 2017.03.06															*/
/* 	Author:    Anita										   						*/
/*----------------------------------------------------------------------------------*/

/************************************************************************************/	
/* 	Include file																	*/
/************************************************************************************/
#include "stdafx.h"
#include "SinInterfaceApi.h"
#include "SinMPayApi.h"

/************************************************************************************/
/* 	Public																			*/
/************************************************************************************/
OVERLAPPED ov;

//-------------------------------------------------   FUNCTION   ------------------------------------------------
//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinInterface_GetApiVersion										//
// Description    : Get API's version.												//
// Input          : None															//
// Output         : Version: API's version string.									//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinInterface_GetApiVersion(LPSHStruct psh, CString *Version)
{
	*Version = _T("Sin HID & COM Series Interface Function V1.11");
	return TRUE;
}

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : AsciiToHex														//
// Description    : ASCII code translate to HEX.									//
// Input          : beLen: Data length before translation.							//
//					beBuf: Data before translation.									//
// Output         : afLen: Data length after translation.							//
//					afBuf: Save translation data.									//
// Return         : None															//
//////////////////////////////////////////////////////////////////////////////////////
void AsciiToHex(INT beLen, BYTE *beBuf, INT *afLen, byte *afBuf)
{
	BYTE tmp[2] = { 0 };
	BYTE BeforeTmp[3000] = { 0 };
	BYTE AfterTmp[3000] = { 0 };
	int iTemp = 0;
	int iCheckSum = 0;	// translate ascii num
	int iLen = beLen;	// Length of szCmdLine
	register int j;

	for (j = 0; j<iLen; j++)
	{
		BeforeTmp[j] = toupper(beBuf[j]);
	}
	j = 0;
	for (register int i = 0; i<iLen; i += 2)
	{
		tmp[0] = BeforeTmp[i];
		tmp[1] = BeforeTmp[i + 1];

		// First, we transfer tmp[1] into Integer-Ascii, then save into iTemp
		if (tmp[1] >= 0x41)			// 'A' ASCII code = 65 = 0x41
		{
			iTemp = tmp[1] - 0x37;	// '7' ASCII code = 55 = 0x37, so (65 - 55) = 10 
		}
		else						// '0' ASCII code = 0x30
		{
			iTemp = tmp[1] - 0x30;	// we want '0' = 0
		}

		// Second, we transfer tmp[0] into Integer-Ascii, then save into iTemp
		if (tmp[0] >= 0x41)
		{
			iTemp += ((int)(tmp[0] - 0x37) * 16);
		}
		else
		{
			iTemp += ((int)(tmp[0] - 0x30) * 16);
		}

		// Now we got the complete ascii num
		AfterTmp[j++] = iTemp;
	}

	// save back the szCmdline
	for (j = 0; j<iLen / 2; j++)
	{
		afBuf[j] = AfterTmp[j];
	}

	*afLen = iLen / 2;
}

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : HexToAscii														//
// Description    : HEX translate to ASCII code.									//
// Input          : beLen: Data length before translation.							//
//					beBuf: Data before translation.									//
// Output         : afLen: Data length after translation.							//
//					afBuf: Save translation data.									//
// Return         : None															//
//////////////////////////////////////////////////////////////////////////////////////
void HexToAscii(UINT beLen, BYTE *beBuf, UINT *afLen, BYTE *afBuf)
{
	BYTE tmp[2] = { 0 };
	BYTE BeforeTmp[2048] = { 0 };
	BYTE AfterTmp[2048] = { 0 };
	int iTemp = 0;

	memcpy(BeforeTmp, beBuf, beLen);

	UINT j = 0;

	for (register UINT i = 0; i<beLen; i++)
	{
		tmp[0] = (BeforeTmp[i] & 0xF0) >> 4;
		tmp[1] = BeforeTmp[i] & 0x0F;

		if (tmp[0] < 0x0A)	tmp[0] += 0x30;
		else				tmp[0] += 0x37;

		if (tmp[1] < 0x0A)	tmp[1] += 0x30;
		else				tmp[1] += 0x37;

		// Now we got the complete ascii num
		AfterTmp[j++] = tmp[0]; 
		AfterTmp[j++] = tmp[1];
	}

	memcpy(afBuf, AfterTmp, j);

	*afLen = j;
}

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : translateCMD													//
// Description    : Translate command.												//
// Input          : bAddEDC:														//
//						True: Calculate EDC and add to command.						//
//						False: Non-added EDC.										//
//					bMask:															//
//						True: Parameter use ASCII Code and don't translate to HEX.	//
//						False: Parameter translate to HEX.							//
// Output         : None															//
// Return         : None															//
//////////////////////////////////////////////////////////////////////////////////////
void translateCMD(LPSHStruct pse, BOOL bAddEDC, BOOL bMask)
{
	BYTE tmp[2] = { 0 };
	BYTE BeforeTmp[1024] = { 0 };
	BYTE AfterTmp[1024] = { 0 };
	int iTemp = 0;
	int iLen = 0;
	int iCheckSum = 0;		// translate ascii num
	int oLen = 0;
	int i = 0;

	if (!bMask)
	{
		//iLen = strlen( (char*)pse->szCmdBuffer );				// Length of szCmdLine
		iLen = 0;
		while (pse->byCmdWriteBuffer[i++])
			iLen++;

		AsciiToHex(iLen, pse->byCmdWriteBuffer, &oLen, AfterTmp);
		memcpy(pse->byCmdWriteBuffer, AfterTmp, oLen);				// save back the szCmdline
		memset(pse->byCmdWriteBuffer + oLen, 0, sizeof(pse->byCmdWriteBuffer) + oLen);
	}
	else
	{
		AsciiToHex(10, pse->byCmdWriteBuffer, &oLen, AfterTmp);
		iLen = (AfterTmp[1] * 256) + AfterTmp[2];
		memcpy(pse->byCmdWriteBuffer, AfterTmp, oLen);				// save back the szCmdline
		memcpy(pse->byCmdWriteBuffer + 5, pse->byCmdWriteBuffer + 10, iLen - 2);
		oLen = iLen + 3;
		memset(pse->byCmdWriteBuffer + oLen, 0, 1024 - oLen);
	}

	// handle EDC
	if (bAddEDC) // if EDC flag is TRUE
	{
		INT i;
		// Calculate Checksum
		for (i = 0; i<oLen/*iLen/2*/; i++)
		{
			iCheckSum ^= pse->byCmdWriteBuffer[i];
		}
		// Fill checksum into the end of szCmdline
		pse->byCmdWriteBuffer[i] = iCheckSum;
		pse->dwCmdWriteLength = oLen + (int)1;
	}
	else
	{
		pse->dwCmdWriteLength = oLen;//iLen/2;
	}
}

//-------------------------------------------------   HID SERIES FUNCTION   ------------------------------------------------
//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : GetDeviceCapabilities											//
// Description    : Get device capabilities.										//
// Input          : None															//
// Output         : None															//
// Return         : None															//
//////////////////////////////////////////////////////////////////////////////////////
VOID GetDeviceCapabilities(LPSHStruct psh)
{
	PHIDP_PREPARSED_DATA	PreparsedData;

	// Get the device's functions to a buffer struture of PHIDP_PREPARSED_DATA.
	HidD_GetPreparsedData(psh->hDeviceHandle, &PreparsedData);

	// Get the device's capabilities.
	HidP_GetCaps(PreparsedData, &psh->Capabilities);

	// Release the buffer struture.
	HidD_FreePreparsedData(PreparsedData);
}

//////////////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinHid_GetDevice														//
// Description    : Find the HID device which matches Singular's Vendor ID and Product ID.	//										//
// Input          : VendorID: 0x273A														//
//					ProductID: 0x0131														//
// Output         : None																	//
// Return         : True or False															//
//////////////////////////////////////////////////////////////////////////////////////////////
BOOL SinHid_GetDevice(LPSHStruct psh, INT VendorID, INT ProductID, INT index)
{
	HIDD_ATTRIBUTES						Attributes;
	SP_DEVICE_INTERFACE_DATA			devInfoData;
	HANDLE								hDevInfo;

	BOOL								LastDevice = FALSE;
	INT									MemberIndex = 0;
	LONG								Result;
	CString								UsageDescription;
	GUID								HidGuid;
	ULONG								Length;
	ULONG								Required;

	INT									iCount=0;

	Length = 0;
	psh->detailData = NULL;
	psh->hDeviceHandle = NULL;

	// Get all HID devices' GUIDs
	HidD_GetHidGuid(&HidGuid);

	// Get device Handle
	hDevInfo = SetupDiGetClassDevs(&HidGuid, NULL, NULL, DIGCF_PRESENT | DIGCF_INTERFACEDEVICE);
	devInfoData.cbSize = sizeof(devInfoData);

	MemberIndex = 0;
	LastDevice = FALSE;

	do
	{
		// Get device Handle's structure of SP_DEVICE_INTERFACE_DATA
		Result = SetupDiEnumDeviceInterfaces(hDevInfo, 0, &HidGuid, MemberIndex, &devInfoData);

		if (Result != 0)
		{
			//Get the Length value.
			//The call will return with a "buffer too small" error which can be ignored.
			Result = SetupDiGetDeviceInterfaceDetail(hDevInfo, &devInfoData, NULL, 0, &Length, NULL);

			//Allocate memory for the hDevInfo structure, using the returned Length.
			psh->detailData = (PSP_DEVICE_INTERFACE_DETAIL_DATA)malloc(Length);

			//Set cbSize in the detailData structure.
			psh->detailData->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);

			//Call the function again, this time passing it the returned buffer size.
			Result = SetupDiGetDeviceInterfaceDetail(hDevInfo, &devInfoData, psh->detailData, Length, &Required, NULL);

			// Open a device Handle to be read and write.
			psh->hDeviceHandle = CreateFile(psh->detailData->DevicePath,
				GENERIC_WRITE | GENERIC_READ,
				FILE_SHARE_READ | FILE_SHARE_WRITE,
				(LPSECURITY_ATTRIBUTES)NULL,
				OPEN_EXISTING,
				0, // 2011 Alan//FILE_FLAG_OVERLAPPED
				NULL);

			Attributes.Size = sizeof(Attributes);

			//Get this device's Attributes 
			Result = HidD_GetAttributes(psh->hDeviceHandle, &Attributes);

			//Check if this device is what we want to connect.
			psh->MyDeviceDetected = FALSE;
			if (Attributes.VendorID == VendorID)
			{
				if (Attributes.ProductID == ProductID)
				{
					//Both the Vendor ID and Product ID match.
					if (iCount == index) {
						psh->MyDeviceDetected = TRUE;
					}
					iCount++;
					
					psh->MyDevicePathName = psh->detailData->DevicePath;

					psh->m_hReadHandle = CreateFile(psh->MyDevicePathName,
						GENERIC_READ,
						FILE_SHARE_READ | FILE_SHARE_WRITE,
						NULL,
						OPEN_EXISTING,
						FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED,
						NULL);
					if (psh->m_hReadHandle == INVALID_HANDLE_VALUE)
					{
						AfxMessageBox(_T("INVALID_HANDLE_VALUE"));
					}
					else
					{
						psh->isOverlapped = TRUE;	// 2016.06.20 by Anita (overlapped mode turn-on)
						psh->evQuit.ResetEvent();
						// } Read 
						psh->pReadThread = AfxBeginThread(SinHid_ReadThread,
							psh,
							THREAD_PRIORITY_NORMAL,
							0,
							CREATE_SUSPENDED,
							NULL);
						if (psh->pReadThread == NULL)
						{
							CString strLog;
							strLog = "д˵2̲߳ɹ!";
							::AfxMessageBox(strLog);
							CloseHandle(psh->hDeviceHandle);
							psh->hDeviceHandle = INVALID_HANDLE_VALUE;
							exit(0);
						}
						psh->pReadThread->m_bAutoDelete = FALSE;
						psh->pReadThread->ResumeThread();
					}

					//Register to receive device notifications.
					//RegisterForDeviceNotifications();

					//Get the device's capablities.
					GetDeviceCapabilities(psh);
				}
				else
					CloseHandle(psh->hDeviceHandle);
			}
			else
				CloseHandle(psh->hDeviceHandle);

			free(psh->detailData);
		}
		else
		{
			//SetupDiEnumDeviceInterfaces returned 0, so there are no more devices to check.
			LastDevice = TRUE;
		}
		//If we haven't found the device yet, and haven't tried every available device,
		//try the next one.
		MemberIndex = MemberIndex + 1;

	} while ((LastDevice == FALSE) && (psh->MyDeviceDetected == FALSE));

	//Free the memory reserved for hDevInfo by SetupDiClassDevs.
	SetupDiDestroyDeviceInfoList(hDevInfo);

	return psh->MyDeviceDetected;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinHid_SwitchReadHandle															//
// Description    : Switch ReadHandle.																//
// Input          : fOverlapped: ReadHandle setting.												//
//					 True: Close synchronous ReadHandle and then create asynchronous ReadHandle.	//
//					 False: Close asynchronous ReadHandle and then create synchronous ReadHandle.	//
// Output         : None																			//
// Return         : True or False																	//
//////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL SinHid_SwitchReadHandle(LPSHStruct psh, BOOL fOverlapped)
{
	// Close Read Handle
	CloseHandle(psh->m_hReadHandle);
	psh->m_hReadHandle = INVALID_HANDLE_VALUE;
	
	if (!fOverlapped)
	{
		// Close Event
		if (ov.hEvent != NULL) {
			SetEvent(ov.hEvent);
			ov.hEvent = NULL;
		}
		// Close Read Thread Handle
		HANDLE hThread = psh->pReadThread->m_hThread;	//  thread  HANDLE s_
		psh->evQuit.SetEvent();
		DWORD rc = WaitForSingleObject(hThread, INFINITE);
		delete psh->pReadThread;		// ]m_bAutoDeletew]FALSE, ҥHnۦ浽
		psh->pReadThread = NULL;

		// Create Read Handle (Synchronous)
		psh->m_hReadHandle = CreateFile(psh->MyDevicePathName,
			GENERIC_READ,
			FILE_SHARE_READ | FILE_SHARE_WRITE,
			NULL,
			OPEN_EXISTING,
			0,
			NULL);
		if (psh->m_hReadHandle == INVALID_HANDLE_VALUE)
		{
			AfxMessageBox(_T("INVALID_HANDLE_VALUE"));
		}
	}	
	else
	{
		// Create Read Handle (Asynchronous)
		psh->m_hReadHandle = CreateFile(psh->MyDevicePathName,
										GENERIC_READ,
										FILE_SHARE_READ | FILE_SHARE_WRITE,
										NULL,
										OPEN_EXISTING,
										FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED,
										NULL);

		if (psh->m_hReadHandle == INVALID_HANDLE_VALUE)
		{
			AfxMessageBox(_T("INVALID_HANDLE_VALUE"));
		}
		else
		{
			psh->evQuit.ResetEvent();
			// Start Read Thread
			psh->pReadThread = AfxBeginThread(SinHid_ReadThread,
				psh,
				THREAD_PRIORITY_NORMAL,
				0,
				CREATE_SUSPENDED,
				NULL);
			if (psh->pReadThread == NULL)
			{
				CString strLog;
				strLog = "Read Thread Fail!";
				::AfxMessageBox(strLog);
				return FALSE;
			}
			else
			{
				psh->pReadThread->m_bAutoDelete = FALSE;
				psh->pReadThread->ResumeThread();
			}		
		}
	}
	if(psh->m_hReadHandle != INVALID_HANDLE_VALUE)
	{				
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinHid_CloseDevice												//
// Description    : Close device handle.											//
// Input          : None															//
// Output         : None															//
// Return         : None															//
//////////////////////////////////////////////////////////////////////////////////////
VOID SinHid_CloseDevice(LPSHStruct psh)
{
	psh->isOverlapped = FALSE;	
	//Close Read Handle
	if (psh->m_hReadHandle != INVALID_HANDLE_VALUE) {
		CloseHandle(psh->m_hReadHandle);
		psh->m_hReadHandle = INVALID_HANDLE_VALUE;
	}
	//Close Event
	if (ov.hEvent != NULL) {
		SetEvent(ov.hEvent);
		ov.hEvent = NULL;
	}
	// Close Read Thread Handle
	HANDLE hThread = psh->pReadThread->m_hThread;	//  thread  HANDLE s_
	psh->evQuit.SetEvent();
	DWORD rc = WaitForSingleObject(hThread, INFINITE);
	delete psh->pReadThread;		// ]m_bAutoDeletew]FALSE, ҥHnۦ浽
	psh->pReadThread = NULL;
	//Close Device Handle
	if (psh->hDeviceHandle != INVALID_HANDLE_VALUE) {
		CloseHandle(psh->hDeviceHandle);
		psh->hDeviceHandle = INVALID_HANDLE_VALUE;
	}
}

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinHid_Write													//
// Description    : Translate command, and then use WriteFile to send command.		//
// Input          : bAddEDC:														//
//						True: Calculate EDC and add to command.						//
//						False: Non-added EDC.										//
//					bMaskParameter:													//
//						True: Parameter use ASCII Code and don't translate to HEX.	//
//						False: Parameter translate to HEX.							//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinHid_Write( LPSHStruct psh, BOOL bAddEDC, BOOL bMaskParameter)
{
	DWORD BytesWrite = 0;
	BOOL Status = false;
	int nLen = 0;
	int i = 0, j = 0, k = 0;

	translateCMD(psh, bAddEDC, bMaskParameter);

	if (psh->hDeviceHandle != INVALID_HANDLE_VALUE)
	{
		if ((int)psh->dwCmdWriteLength % 64 != 0)		//->Packet
			nLen = (int)psh->dwCmdWriteLength / 64 + 1;
		else
			nLen = (int)psh->dwCmdWriteLength / 64;

		psh->OutputReport[0] = 0;	//Packet Head = [0x00]
		for (i = 0; i<nLen; i++)
		{
			for (j = 0; j<64; j++)	//1 Packet = 64 Byte
			{
				psh->OutputReport[j + 1] = psh->byCmdWriteBuffer[k++];
			}
			Status = WriteFile(psh->hDeviceHandle, psh->OutputReport, psh->Capabilities.OutputReportByteLength, &BytesWrite, NULL);
		}
		if (Status)	return true;
		else		return false;
	}
	else return false;
}

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinHid_SingleRead												//
// Description    : Use ReadFile to get response for synchronous.					//
// Input          : None															//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinHid_SingleRead(LPSHStruct psh)
{
	DWORD dwRes = 0, BytesRead = 0;
	INT i = 0;
	BOOL bStatus = FALSE;
	UINT iCmdIndex = 0, iPackageCount = 0;

	while (true)
	{
		memset(psh->InputReport, 0, sizeof(psh->InputReport));
		bStatus = ReadFile(psh->m_hReadHandle,
						   psh->InputReport,
						   psh->Capabilities.InputReportByteLength,
						   &BytesRead,
						   0);

		if (bStatus)
		{
			if (BytesRead != 0)
			{
				iPackageCount++;
				if (iPackageCount == 1)
				{
					// check Respond Head
					if ((psh->InputReport[1] == 0x1C) || (psh->InputReport[1] == 0x2C))
					{
						memset(psh->byCmdReadBuffer, 0, sizeof(psh->byCmdReadBuffer));
						// get ack length
						psh->dwCmdReadLength = psh->InputReport[2] * 256 + psh->InputReport[3];
						// get first package data
						for (i = 0; i < psh->Capabilities.InputReportByteLength - 4; i++)
						{
							psh->byCmdReadBuffer[iCmdIndex++] = psh->InputReport[i + 4];	//Packet Head must be [0x00]
							if (iCmdIndex >= psh->dwCmdReadLength) {
								iCmdIndex = 0;
								iPackageCount = 0;
								return true;
							}
						}
					}
					// Debug message
					else if (psh->InputReport[1] == 0x0C)
					{
						memset(psh->byCmdReadBuffer, 0, sizeof(psh->byCmdReadBuffer));
						// get ack length
						psh->dwCmdReadLength = psh->InputReport[2] * 256 + psh->InputReport[3] + 3;	//+3: load all response
						// get first package data
						for (i = 0; i < psh->Capabilities.InputReportByteLength - 1; i++)
						{
							psh->byCmdReadBuffer[iCmdIndex++] = psh->InputReport[i + 1];		//Packet Head must be [0x00]
							if (iCmdIndex >= psh->dwCmdReadLength) {
								iCmdIndex = 0;
								iPackageCount = 0;
								return true;
							}
						}
					}
					// Data format error
					else
					{
						iCmdIndex = 0;
						iPackageCount = 0;
						return FALSE;
					}
				}
				// (Package > 1)
				else
				{
					// get next package data
					for (i = 0; i < psh->Capabilities.InputReportByteLength - 1; i++)
					{
						psh->byCmdReadBuffer[iCmdIndex++] = psh->InputReport[i + 1];	//Packet Head must be [0x00]
						if (iCmdIndex >= psh->dwCmdReadLength) {
							iCmdIndex = 0;
							iPackageCount = 0;
							return true;
						}
					}
				}
			}
		}
		else
		{
			// Error in communications; report it.
			return false;
		}
	}
}

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinHid_Read														//
// Description    : Use ReadFile to get response for asynchronous.					//
// Input          : None															//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinHid_Read(LPSHStruct psh)
{
	DWORD dwRes = 0, BytesRead = 0;
	INT i = 0;
	BOOL bStatus = FALSE;
	UINT iCmdIndex = 0, iPackageCount = 0;
	
	if (psh->m_hReadHandle != INVALID_HANDLE_VALUE)
	{
		while (true)
		{
			ResetEvent(ov.hEvent);
			memset(psh->InputReport, 0, sizeof(psh->InputReport));
			bStatus = ReadFile(psh->m_hReadHandle,
							   psh->InputReport,
							   psh->Capabilities.InputReportByteLength,
							   &BytesRead,
							   &ov);

			if (!bStatus)
			{
				// Timeout setting : 10s
				if (WaitForSingleObject(ov.hEvent, 10000) != WAIT_OBJECT_0){
					return FALSE;
				}

				bStatus = GetOverlappedResult(psh->m_hReadHandle,
												&ov,
												&BytesRead,
												FALSE);

				if (bStatus)
				{
					if (BytesRead != 0)
					{
						iPackageCount++;
						if (iPackageCount == 1)
						{
							// check Respond Head
							if ((psh->InputReport[1] == 0x1C) || (psh->InputReport[1] == 0x2C))
							{
								memset(psh->byCmdReadBuffer, 0, sizeof(psh->byCmdReadBuffer));
								// get ack length
								psh->dwCmdReadLength = psh->InputReport[2] * 256 + psh->InputReport[3];
								// get first package data
								for (i = 0; i < psh->Capabilities.InputReportByteLength - 4; i++)
								{
									psh->byCmdReadBuffer[iCmdIndex++] = psh->InputReport[i + 4];	//Packet Head must be [0x00]
									if (iCmdIndex >= psh->dwCmdReadLength) {
										iCmdIndex = 0;
										iPackageCount = 0;
										return true;
									}
								}
							}
							// Debug message
							else if (psh->InputReport[1] == 0x0C)
							{
								memset(psh->byCmdReadBuffer, 0, sizeof(psh->byCmdReadBuffer));
								// get ack length
								psh->dwCmdReadLength = psh->InputReport[2] * 256 + psh->InputReport[3] + 3;	//+3: load all response
								// get first package data
								for (i = 0; i < psh->Capabilities.InputReportByteLength - 1; i++)
								{
									psh->byCmdReadBuffer[iCmdIndex++] = psh->InputReport[i + 1];		//Packet Head must be [0x00]
									if (iCmdIndex >= psh->dwCmdReadLength) {
										iCmdIndex = 0;
										iPackageCount = 0;
										return true;
									}
								}
							}
							// Data format error
							else
							{
								iCmdIndex = 0;
								iPackageCount = 0;
								return FALSE;
							}
						}
						// (Package > 1)
						else
						{
							// get next package data
							for (i = 0; i < psh->Capabilities.InputReportByteLength - 1; i++)
							{
								psh->byCmdReadBuffer[iCmdIndex++] = psh->InputReport[i + 1];	//Packet Head must be [0x00]
								if (iCmdIndex >= psh->dwCmdReadLength) {
									iCmdIndex = 0;
									iPackageCount = 0;
									return true;
								}
							}
						}
					}
				}
				else
				{
					// Error in communications; report it.
					return false;
				}
			}
			// ReadFile = TRUE
			else
			{
				iPackageCount++;
				if (iPackageCount == 1)
				{
					// check Respond Head
					if ((psh->InputReport[1] == 0x1C) || (psh->InputReport[1] == 0x2C))
					{
						memset(psh->byCmdReadBuffer, 0, sizeof(psh->byCmdReadBuffer));
						// get ack length
						psh->dwCmdReadLength = psh->InputReport[2] * 256 + psh->InputReport[3];
						// get first package data
						for (i = 0; i < psh->Capabilities.InputReportByteLength - 4; i++)
						{
							psh->byCmdReadBuffer[iCmdIndex++] = psh->InputReport[i + 4];	//Packet Head must be [0x00]
							if (iCmdIndex >= psh->dwCmdReadLength) {
								iCmdIndex = 0;
								iPackageCount = 0;
								return true;
							}
						}
					}
					// Debug message
					else if (psh->InputReport[1] == 0x0C)
					{
						memset(psh->byCmdReadBuffer, 0, sizeof(psh->byCmdReadBuffer));
						// get ack length
						psh->dwCmdReadLength = psh->InputReport[2] * 256 + psh->InputReport[3] + 3;	//+3: load all response
						// get first package data
						for (i = 0; i < psh->Capabilities.InputReportByteLength - 1; i++)
						{
							psh->byCmdReadBuffer[iCmdIndex++] = psh->InputReport[i + 1];		//Packet Head must be [0x00]
							if (iCmdIndex >= psh->dwCmdReadLength) {
								iCmdIndex = 0;
								iPackageCount = 0;
								return true;
							}
						}
					}
					// Data format error
					else
					{
						iCmdIndex = 0;
						iPackageCount = 0;
						return FALSE;
					}
				}
				// (Package > 1)
				else
				{
					// get next package data
					for (i = 0; i < psh->Capabilities.InputReportByteLength - 1; i++)
					{
						psh->byCmdReadBuffer[iCmdIndex++] = psh->InputReport[i + 1];	//Packet Head must be [0x00]
						if (iCmdIndex >= psh->dwCmdReadLength) {
							iCmdIndex = 0;
							iPackageCount = 0;
							return true;
						}
					}
				}
			}
		}
	}
	else
		return false;

}

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinHid_ReadThread												//
// Description    : Read thread for asynchronous.									//
// Input          : None															//
// Output         : None															//
// Return         : 0																//
//////////////////////////////////////////////////////////////////////////////////////
UINT SinHid_ReadThread(LPVOID lpParam)
{
	//  Struct ܼƱi
	LPSHStruct psh = (LPSHStruct)lpParam;

	BOOL bSuccess = FALSE;

	if ( ov.hEvent != NULL ) {
		ResetEvent(ov.hEvent);
	}
	else {
		ov.hEvent = CreateEvent(NULL,   // default security attributes 
								TRUE,   // manual-reset event 
								FALSE,  // not signaled 
								NULL);	// no name
	}
	// Initialize the rest of the OVERLAPPED structure to zero.
	ov.Internal = 0;
	ov.InternalHigh = 0;
	ov.Offset = 0;
	ov.OffsetHigh = 0;

	while ( psh->m_hReadHandle != INVALID_HANDLE_VALUE )	// Start Read-loop, until Handel close
	{
		if (psh->m_hReadHandle == INVALID_HANDLE_VALUE) {
			break;
		}
		else
		{	// T{O_}DPB
			if ( psh->isOverlapped ) {
			//	OutputDebugString(_T("A"));
				bSuccess = SinHid_Read(psh);
				if(bSuccess) {
					SinMPay_DataComming(psh, Type_HID);
				}
			}
		/*	if ( WaitForSingleObject(ov.hEvent, 0) == WAIT_OBJECT_0 ) {
				ResetEvent(psh->m_hReadHandle);
			}*/
			// Wait Read Thread Quit
			if (WaitForSingleObject(psh->evQuit, 0) == WAIT_OBJECT_0) {
				break;
			}
		}
	}
	return 0;
}

//-------------------------------------------------   COM SERIES FUNCTION   ------------------------------------------------
//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinComGetandSetTimeouts											//
// Description    : Get and set timeouts for COM Port.								//
// Input          : dwReadInter: Read interval timeout.								//
//					dwReadConstant: Read total timeout constant.					//
//					dwReadMultiplier: Read total timeout multiplier.				//
//					dwWriteConstant: Write total timeout constant.					//
//					dwWriteMultiplier: Write total timeout multiplier.				//
//					(0 ~ MAXDWORD milliseconds.)										//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinComGetandSetTimeouts(
	LPSHStruct psh,
	DWORD dwReadInter = 100,
	DWORD dwReadConstant = 200,
	DWORD dwReadMultiplier = 0,	//200
	DWORD dwWriteConstant = 100,
	DWORD dwWriteMultiplier = 100
	)
{
	if (psh->hDeviceHandle == INVALID_HANDLE_VALUE)
		return FALSE;

	BOOL bSuccess = FALSE;

	// Read timeouts
	COMMTIMEOUTS cto;
	bSuccess = GetCommTimeouts(psh->hDeviceHandle, &cto);
	if (!bSuccess)
	{
		return FALSE;
	}

	// fill in our data
	cto.ReadIntervalTimeout = dwReadInter;
	cto.ReadTotalTimeoutConstant = dwReadConstant;
	cto.ReadTotalTimeoutMultiplier = dwReadMultiplier;
	cto.WriteTotalTimeoutConstant = dwWriteConstant;
	cto.WriteTotalTimeoutMultiplier = dwWriteMultiplier;

	// Write timeouts
	bSuccess = SetCommTimeouts(psh->hDeviceHandle, &cto);

	return (bSuccess ? TRUE : FALSE);
}

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinCom_Open														//
// Description    : Create device handle and set com port.							//
// Input          : dwComNum: Com number. 1: COM1 ~ 30: COM30						//
//					dwBaudrate: Set baudrate. 9600, 19200, 115200 (default)			//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinCom_Open(LPSHStruct psh, DWORD dwComNum, DWORD dwBaudrate)
{
	// Check Com Number 
	if (dwComNum <= 0)
	{
		return FALSE;
	}

	BOOL bSuccess = FALSE;
	memset(psh->byCmdReadBuffer, 0, sizeof(psh->byCmdReadBuffer));

	TCHAR ComDevice[16];

	_stprintf_s(ComDevice, _T("\\\\.\\COM%d"), dwComNum); // "\\\\.\\COM10" address of name of the communications device

	psh->hDeviceHandle = CreateFile(ComDevice,						// pointer to name of the file
									GENERIC_READ | GENERIC_WRITE,	// access mode
									0,								// comm devices must be opened w/exclusive-access
									NULL,							// no security attributs
									OPEN_EXISTING,					// comm devices must use OPEN_EXISTING
									FILE_FLAG_OVERLAPPED,			// overlapped I/O
									NULL);							// hTemplate must be NULL for comm devices 
	Sleep(10);
	if (psh->hDeviceHandle == INVALID_HANDLE_VALUE)
		return FALSE;

	// Read DCB		
	DCB dcb;
	bSuccess = GetCommState(psh->hDeviceHandle, &dcb);
	Sleep(10);
	if (!bSuccess)	return FALSE;

	// fill in our data	
	dcb.DCBlength = sizeof(DCB);
	dcb.BaudRate = dwBaudrate;
	dcb.ByteSize = 8;
	dcb.Parity = NOPARITY;
	dcb.StopBits = ONESTOPBIT;
	/*
	dcb.fBinary = TRUE;             // Binary mode; no EOF check
	dcb.fParity = TRUE;				// None parity checking.
	dcb.fOutxCtsFlow = FALSE;       // No CTS output flow control
	dcb.fOutxDsrFlow = FALSE;       // No DSR output flow control
	dcb.fDtrControl = DTR_CONTROL_ENABLE; // DTR flow control type
	dcb.fDsrSensitivity = FALSE;    // DSR sensitivity
	dcb.fTXContinueOnXoff = TRUE;   // XOFF continues Tx
	dcb.fOutX = FALSE;              // No XON/XOFF out flow control
	dcb.fInX = FALSE;               // No XON/XOFF in flow control
	dcb.fErrorChar = FALSE;         // Disable error replacement.x
	dcb.fNull = FALSE;              // Disable null stripping.
	dcb.fRtsControl = RTS_CONTROL_ENABLE; // RTS flow control
	dcb.fAbortOnError = FALSE;      // Do not abort reads/writes on error.
	*/

	// Write DCB
	bSuccess = SetCommState(psh->hDeviceHandle, &dcb);
	if (!bSuccess)	return FALSE;

	// Set Timeouts
	bSuccess = SinComGetandSetTimeouts(psh, 1, 2, 2, 1, 1);
	if (!bSuccess)	return FALSE;

	// Clear com port
	PurgeComm(psh->hDeviceHandle, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR);

	// Begin Thread
	psh->pReadThread = AfxBeginThread(SinCom_ReadThreadFunction, psh, 0, 0, 0, NULL);

	return TRUE;
}
//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinCom_Write													//
// Description    : Translate command, and then use WriteFile to send command.		//
// Input          : bAddEDC:														//
//						True: Calculate EDC and add to command.						//
//						False: Non-added EDC.										//
//					bMaskParameter:													//
//						True: Parameter use ASCII Code and don't translate to HEX.	//
//						False: Parameter translate to HEX.							//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinCom_Write(LPSHStruct psh, BOOL bAddEDC, BOOL bMaskParameter)
{
	BOOL bSuccess = FALSE;
	DWORD dwBytesWritten;

	translateCMD(psh, bAddEDC, bMaskParameter);

	bSuccess = WriteFile(psh->hDeviceHandle,
						 psh->byCmdWriteBuffer,
						 psh->dwCmdWriteLength,
						 &dwBytesWritten,
						 &ov);

	if (!bSuccess && (GetLastError() == ERROR_IO_PENDING))
	{
		// Check Overlapped Result
		WaitForSingleObject(psh->hDeviceHandle, 50);	//timeout: 100ms->50ms
		bSuccess = GetOverlappedResult(psh->hDeviceHandle, &ov, &dwBytesWritten, TRUE);
	}
	return (bSuccess ? TRUE : FALSE);
}
//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinCom_Read														//
// Description    : Use ReadFile to get response.									//
// Input          : None															//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinCom_Read(LPSHStruct psh)
{
	DWORD dwBytes = 0;
	DWORD dwBytesRead = 0;
	ResetEvent(ov.hEvent);
	memset(psh->byCmdReadBuffer, 0, sizeof(psh->byCmdReadBuffer));
	BOOL bSuccess = ReadFile(psh->hDeviceHandle,			// ComDevice handle
							 psh->byCmdReadBuffer,			// Data to write		
							 sizeof(psh->byCmdReadBuffer),	// length prepare to read
							 &dwBytesRead,					// length finally be read
							 &ov);							// overlapped IO

	if (!bSuccess && (GetLastError() == ERROR_IO_PENDING))
	{
		// Timeout setting : 10s
		if (WaitForSingleObject(ov.hEvent, 10000) != WAIT_OBJECT_0){
			return FALSE;
		}
		if (GetOverlappedResult(psh->hDeviceHandle, &ov, &dwBytesRead, FALSE))
		{
			if (dwBytesRead != 0)
				bSuccess = TRUE;
		}
		else
			bSuccess = FALSE;
	}

	return (bSuccess ? TRUE : FALSE);
}
//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinCom_ReadThreadFunction										//
// Description    : Read thread for asynchronous.									//
// Input          : None															//
// Output         : None															//
// Return         : 0																//
//////////////////////////////////////////////////////////////////////////////////////
UINT SinCom_ReadThreadFunction(LPVOID pParam)
{
	LPSHStruct psh = (LPSHStruct)pParam;

	BOOL bSuccess = FALSE;

	if (ov.hEvent != NULL) {
		ResetEvent(ov.hEvent);
	}
	else {
		ov.hEvent = CreateEvent(NULL,  // default security attributes 
								TRUE,  // manual-reset event 
								FALSE, // not signaled 
								NULL); // no name
	}

	// Initialize the rest of the OVERLAPPED structure to zero.
	ov.Internal = 0;
	ov.InternalHigh = 0;
	ov.Offset = 0;
	ov.OffsetHigh = 0;

	psh->isOverlapped = TRUE;
	while (psh->hDeviceHandle != INVALID_HANDLE_VALUE)	// Start Read-loop, until com close
	{
		if (psh->isOverlapped)
		{
			bSuccess = SinCom_Read(psh);
			if (bSuccess) {
				SinMPay_DataComming(psh, Type_COM);
			}
		}
		Sleep(50);
	}
	return 0;
}
//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinCom_Close													//
// Description    : Close device handle.											//
// Input          : None															//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinCom_Close(LPSHStruct psh)
{
	BOOL bSuccess = FALSE;

	if (ov.hEvent != NULL) {
		SetEvent(ov.hEvent);
		ov.hEvent = NULL;
	}

	if (psh->hDeviceHandle != INVALID_HANDLE_VALUE)
	{
		bSuccess = CloseHandle(psh->hDeviceHandle);
		Sleep(100);
		psh->hDeviceHandle = INVALID_HANDLE_VALUE;
	}

	return (bSuccess ? TRUE : FALSE);
}

/************************************************************************************/
/* 	History																			*/
/************************************************************************************/
/* SinInterfaceApi V1.0(2017.02.06)
1. from SinHidApi and SinComApi 
2. HIDCOMX */

/* SinInterfaceApi V1.01(2017.02.14)
1. from SinInterfaceApi V1.0 
2. W[{ϥθT */

/* SinInterfaceApi V1.1(2017.02.17) by Alan
1. from SinInterfaceApi V1.01
2. SinHid_GetDeviceW[indexJ, iܬۦPVIDPIDt~@x˸ms(Index̸˸msǱƧ)*/

/* SinInterfaceApi V1.11(2017.03.06) by Anita
1. from SinInterfaceApi V1.1
2. SinCom_Write()timeoutɶ(100msאּ50ms), UDPBROɵeyɶiYu*/
