/*----------------------------------------------------------------------------------*/
/*	Function:  Sin HID & COM Series Interface Function								*/
/* 	Version:   V1.11											   					*/
/*	Last Date: 2017.03.06															*/
/* 	Author:    Anita										   						*/
/*----------------------------------------------------------------------------------*/
#ifndef _SININTERFACEAPI_H_ 
#define _SININTERFACEAPI_H_
/************************************************************************************/	
/* 	Include file																	*/
/************************************************************************************/
extern "C"
{
#include "wtypes.h"
#include "initguid.h"
#include "setupapi.h"
#include "Hidsdi.h"
#include "dbt.h"
}

typedef struct
{
	HIDP_CAPS							Capabilities;
	PSP_DEVICE_INTERFACE_DETAIL_DATA	detailData;

	BOOL		MyDeviceDetected;
	CString		MyDevicePathName;

	HANDLE hDeviceHandle;			// handle for Device
	HANDLE m_hReadHandle;			// haldel for Device read

	HWND hTargetWnd;				// handle for message

	BOOL isOverlapped;				// check use overlapped or direct mehtod

	INT	VendorID;					// Device Vendor ID
	INT	ProductID;					// Device Product ID

	UCHAR OutputReport[65];			// buffer for endpoint transmission
	UCHAR InputReport[65];			// buffer for endpoint receive

	DWORD dwCmdWriteLength;			// Length of command transmission 
	BYTE byCmdWriteBuffer[3000];	// buffer for command transmission

	DWORD dwCmdReadLength;			// Length of response data
	BYTE byCmdReadBuffer[3000];		// buffer for response data

	CString csDataComming;			// Response Data String

	CWinThread* pReadThread;		// Begin Read Thread
	CEvent evQuit;					// Wait for Read Thread Quit

} SHStruct, *PSHStruct, *LPSHStruct;

//-------------------------------------------------   FUNCTION   ------------------------------------------------
//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinInterface_GetApiVersion										//
// Description    : Get API's version.												//
// Input          : None															//
// Output         : Version: API's version string.									//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinInterface_GetApiVersion(LPSHStruct psh, CString *Version);

// HID Series
//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : GetDeviceCapabilities											//
// Description    : Get device capabilities.										//
// Input          : None															//
// Output         : None															//
// Return         : None															//
//////////////////////////////////////////////////////////////////////////////////////
VOID GetDeviceCapabilities(LPSHStruct psh);

//////////////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinHid_GetDevice														//
// Description    : Find the HID device which matches Singular's Vendor ID and Product ID.	//
// Input          : VendorID: 0x273A														//
//					ProductID: 0x0131                                                       //
//                  index: Device number                                                    //
// Output         : None																	//
// Return         : True or False															//
//////////////////////////////////////////////////////////////////////////////////////////////
BOOL SinHid_GetDevice(LPSHStruct psh, INT VendorID, INT ProductID, INT index);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinHid_CloseDevice												//
// Description    : Close device handle.											//
// Input          : None															//
// Output         : None															//
// Return         : None															//
//////////////////////////////////////////////////////////////////////////////////////
VOID SinHid_CloseDevice(LPSHStruct psh);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinHid_Read														//
// Description    : Use ReadFile to get response for asynchronous.					//
// Input          : None															//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinHid_Read(LPSHStruct psh);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinHid_SingleRead												//
// Description    : Use ReadFile to get response for synchronous.					//
// Input          : None															//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinHid_SingleRead(LPSHStruct psh);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinHid_ReadThread												//
// Description    : Read thread for asynchronous.									//
// Input          : None															//
// Output         : None															//
// Return         : 0																//
//////////////////////////////////////////////////////////////////////////////////////
UINT SinHid_ReadThread(LPVOID lpParam);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinHid_Write													//
// Description    : Translate command, and then use WriteFile to send command.		//
// Input          : bAddEDC:														//
//						True: Calculate EDC and add to command.						//
//						False: Non-added EDC.										//
//					bMaskParameter:													//
//						True: Parameter use ASCII Code and don't translate to HEX.	//
//						False: Parameter translate to HEX.							//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinHid_Write(LPSHStruct psh, BOOL bAddEDC, BOOL bMaskParameter);

//////////////////////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinHid_SwitchReadHandle															//
// Description    : Switch ReadHandle.																//
// Input          : fOverlapped: ReadHandle setting.												//
//					 True: Close synchronous ReadHandle and then create asynchronous ReadHandle.	//
//					 False: Close asynchronous ReadHandle and then create synchronous ReadHandle.	//
// Output         : None																			//
// Return         : True or False																	//
//////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL SinHid_SwitchReadHandle(LPSHStruct psh, BOOL fOverlapped);

// COM Series
//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinComGetandSetTimeouts											//
// Description    : Get and set timeouts for COM Port.								//
// Input          : dwReadInter: Read interval timeout.								//
//					dwReadConstant: Read total timeout constant.					//
//					dwReadMultiplier: Read total timeout multiplier.				//
//					dwWriteConstant: Write total timeout constant.					//
//					dwWriteMultiplier: Write total timeout multiplier.				//
//					(0 ~ MAXDWORD milliseconds)										//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinComGetandSetTimeouts(LPSHStruct psh,
							 DWORD dwReadInter,
							 DWORD dwReadConstant,
							 DWORD dwReadMultiplier,
							 DWORD dwWriteConstant,
							 DWORD dwWriteMultiplier);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinCom_Open														//
// Description    : Create device handle and set com port.							//
// Input          : dwComNum: Com number. 1: COM1 ~ 30: COM30						//
//					dwBaudrate: Set baudrate. 9600, 19200, 115200 (default)			//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinCom_Open(LPSHStruct psh, DWORD dwComNum, DWORD dwBaudrate);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinCom_Close													//
// Description    : Close device handle.											//
// Input          : None															//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinCom_Close(LPSHStruct psh);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinCom_Read														//
// Description    : Use ReadFile to get response.									//
// Input          : None															//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinCom_Read(LPSHStruct psh);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinCom_ReadThreadFunction										//
// Description    : Read thread for asynchronous.									//
// Input          : None															//
// Output         : None															//
// Return         : 0																//
//////////////////////////////////////////////////////////////////////////////////////
UINT SinCom_ReadThreadFunction(LPVOID pParam);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinCom_Write													//
// Description    : Translate command, and then use WriteFile to send command.		//
// Input          : bAddEDC:														//
//						True: Calculate EDC and add to command.						//
//						False: Non-added EDC.										//
//					bMaskParameter:													//
//						True: Parameter use ASCII Code and don't translate to HEX.	//
//						False: Parameter translate to HEX.							//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinCom_Write(LPSHStruct psh, BOOL bAddEDC, BOOL bMaskParameter);

#endif