/*--------------------------------------------------------------------------------------------------*/
/*	Function:  Sin Mobile Payment Series Application Programming Interface Function					*/
/* 	Version:   V1.1													   								*/
/*	Last Date: 2017.03.13																			*/
/* 	Author:    Anita										   										*/
/*--------------------------------------------------------------------------------------------------*/
#ifndef _SINMPAYAPI_H_ 
#define _SINMPAYAPI_H_

/************************************************************************************/
/* 	Include file																	*/
/************************************************************************************/
#include "SinInterfaceApi.h"

/************************************************************************************/
/* 	Public																			*/
/************************************************************************************/
#define WM_DATACOMING									WM_USER + 5
#define WM_TERMINAL_ACK									WM_USER + 6
#define WM_TERMINAL_RETURN_MAG_CARD_READ				WM_TERMINAL_ACK + 48

typedef enum
{
	Type_HID = 0,
	Type_COM = 1,
}Interface_TYPE;

//-------------------------------------------------   FUNCTION   ------------------------------------------------
//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_GetApiVersion											//
// Description    : Get API's version.												//
// Input          : None															//
// Output         : Version: API's version string.									//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_GetApiVersion(LPSHStruct psh, CString *Version);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_DataComming												//
// Description    : Get device response data. (for asynchronous)					//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
// Output         : None															//
// Return         : None															//
//////////////////////////////////////////////////////////////////////////////////////
VOID SinMPay_DataComming(LPSHStruct psh, Interface_TYPE Type);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_GetTerminalVersion										//
// Description    : Get terminal's version.											//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
// Output         : Version: Version string.										//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_GetTerminalVersion(LPSHStruct psh, Interface_TYPE Type, CString *Version);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_GetReaderSerialNumber									//
// Description    : Get reader's serial number.										//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
// Output         : SerialNo: Reader's serial number.								//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_GetReaderSerialNumber(LPSHStruct psh, Interface_TYPE Type, CString *SerialNo);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_SetReaderSerialNumber									//
// Description    : Write serial number into reader.								//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
//					SerialNo: Reader's serial number.(must be 16 characters)		//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_SetReaderSerialNumber(LPSHStruct psh, Interface_TYPE Type, CString SerialNo);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_GetMagCardData											//
// Description    : Get magnetic card's data.										//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_GetMagCardData(LPSHStruct psh, Interface_TYPE Type);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_GiveUpAction											//
// Description    : Cancel operation.												//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_GiveUpAction(LPSHStruct psh, Interface_TYPE Type);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_IccSelectCard_X											//
// Description    : Select IC card's type.											//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
//					Samx: Set IC card's type.										//
//						 0x00: Smart card (default)									//
//						 0x01: SAM card 1											//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_IccSelectCard_X(LPSHStruct psh, Interface_TYPE Type, INT Samx);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_IccDetect												//
// Description    : Check card insert.												//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
// Output         : isInserted: 													//
//								True: Card inserted.								//
//								False: Not insert.									//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_IccDetect(LPSHStruct psh, Interface_TYPE Type, BOOL *isInserted);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_IccPowerOn												//
// Description    : IC card power-on and reset.										//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
// Output         : Atr: Get IC card's ATR(answer to reset) values.					//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_IccPowerOn(LPSHStruct psh, Interface_TYPE Type, CString *Atr);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_IccPowerOff												//
// Description    : Power-off IC card.												//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_IccPowerOff(LPSHStruct psh, Interface_TYPE Type);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_IccAccess												//
// Description    : Send APDU to IC card, and then get response APDU.				//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
//					Capdu: Write APDU command.										//
// Output         : Rapdu: Read response APDU.										//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_IccAccess(LPSHStruct psh, Interface_TYPE Type, CString Capdu, CString *Rapdu);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_IccGetCardholderInfo									//
// Description    : Get smart card's holder information.							//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
// Output         : Pan: Account number.											//
//					Name: Card's holder name.										//
//					Exp: Expiration date.											//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_IccGetCardholderInfo(LPSHStruct psh, Interface_TYPE Type, CString *Pan, CString *Name, CString *Exp);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_IccGetTrack2Data										//
// Description    : Get smart card's track 2 data.									//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
// Output         : Track2Data: Show the card's track 2 data.						//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_IccGetTrack2Data(LPSHStruct psh, Interface_TYPE Type, CString *Track2Data);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_MemorySelectCardType									//
// Description    : Select memory card's type.										//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
//					CardType: Set memory card's type.								//
//						 0x02: SLE4442												//
//						 0x04: SLE4428												//
//						 0x0A: AT24C01												//
//						 0x0B: AT24C02												//
//						 0x0C: AT24C04												//
//						 0x0D: AT24C08												//
//						 0x0E: AT24C16												//
//						 0x13: AT24C32												//
//						 0x14: AT24C64												//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_MemorySelectCardType(LPSHStruct psh, Interface_TYPE Type, INT CardType);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_MemoryPowerOn											//
// Description    : Memory card power-on and reset.									//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
// Output         : Atr: Get memory card's ATR values.(SLE4442/SLE4428 have ATR.)	//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_MemoryPowerOn(LPSHStruct psh, Interface_TYPE Type, CString *Atr);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_MemoryGetCardType										//
// Description    : Get memory card's type.											//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
// Output         : CardType: Get memory card's type.								//
//						 0x02: SLE4442												//
//						 0x04: SLE4428												//
//						 0x0A: AT24C01												//
//						 0x0B: AT24C02												//
//						 0x0C: AT24C04												//
//						 0x0D: AT24C08												//
//						 0x0E: AT24C16												//
//						 0x13: AT24C32												//
//						 0x14: AT24C64												//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_MemoryGetCardType(LPSHStruct psh, Interface_TYPE Type, INT *CardType);

//////////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_MemoryReadData												//
// Description    : Read memory card's data.											//
// Input          : Type: Select interface type.										//
//						 Type_HID: Use for HID device.									//
//						 Type_COM: Use for Com port device.								//
//					Address: Set read memory card's start address.(String: 0000~FFFF)	//
//					Length: Read data length.(String: 01~FF)							//
// Output         : Data: Get memory card's data.										//
// Return         : True or False														//
//////////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_MemoryReadData(LPSHStruct psh, Interface_TYPE Type, CString Address, CString Length, CString *Data);

//////////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_MemoryWriteData												//
// Description    : Write data to memory card.											//
// Input          : Type: Select interface type.										//
//						 Type_HID: Use for HID device.									//
//						 Type_COM: Use for Com port device.								//
//					Address: Set write memory card's start address.(String: 0000~FFFF)	//
//					Length: Write data length.(String: 01~FF)							//
//					Data: Write data.(String: 1*2 ~ 255*2 words)						//
// Output         : None																//
// Return         : True or False														//
//////////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_MemoryWriteData(LPSHStruct psh, Interface_TYPE Type, CString Address, CString Length, CString Data);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_MemoryPowerOff											//
// Description    : Power-off memory card.											//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_MemoryPowerOff(LPSHStruct psh, Interface_TYPE Type);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_MemoryReadErrorCounter									//
// Description    : Read memory card's error counter. (for SLE4442/SLE4428)			//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
// Output         : ErCount: Get memory card's error counter.						//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_MemoryReadErrorCounter(LPSHStruct psh, Interface_TYPE Type, CString *ErCount);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_MemoryVerifyPsc											//
// Description    : Verify memory card's password. (for SLE4442/SLE4428)			//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
//					Length: PSC length.(SLE4442: 03)(SLE4428: 02)					//
//					Psc: Password.(SLE4442: 3*2 words)(SLE4428: 2*2 words)			//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_MemoryVerifyPsc(LPSHStruct psh, Interface_TYPE Type, CString Length, CString Psc);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_MemoryGetPsc											//
// Description    : Get memory card's password. (for SLE4442/SLE4428)				//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
//					Length: PSC length.(SLE4442: 03)(SLE4428: 02)					//
// Output         : Psc: Password.													//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_MemoryGetPsc(LPSHStruct psh, Interface_TYPE Type, CString Length, CString *Psc);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_MemoryModifyPsc											//
// Description    : Modify memory card's password. (for SLE4442/SLE4428)			//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
//					Length: PSC length.(SLE4442: 03)(SLE4428: 02)					//
//					Psc: Password.(SLE4442: 3*2 words)(SLE4428: 2*2 words)			//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_MemoryModifyPsc(LPSHStruct psh, Interface_TYPE Type, CString Length, CString Psc);

//////////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_MemoryReadDataWithProtectBit								//
// Description    : Read memory card's data with protect bit. (for SLE4428)				//
// Input          : Type: Select interface type.										//
//						 Type_HID: Use for HID device.									//
//						 Type_COM: Use for Com port device.								//
//					Address: Set read memory card's start address.(String: 0000~FFFF)	//
//					Length: Read data length.(String: 01~FF)							//
// Output         : DataWithPb: Get memory card's data with protect bit.				//
// Return         : True or False														//
//////////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_MemoryReadDataWithProtectBit(LPSHStruct psh, Interface_TYPE Type, CString Address, CString Length, CString *DataWithPb);

//////////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_MemoryWriteDataWithProtectBit								//
// Description    : Write memory card's data with protect bit. (for SLE4428)			//
// Input          : Type: Select interface type.										//
//						 Type_HID: Use for HID device.									//
//						 Type_COM: Use for Com port device.								//
//					Address: Set write memory card's start address.(String: 0000~FFFF)	//
//					Length: Write data length.(String: 01~FF)							//
//					Data: Write data.(String: 1*2 ~ 255*2 words)						//
// Output         : None																//
// Return         : True or False														//
//////////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_MemoryWriteDataWithProtectBit(LPSHStruct psh, Interface_TYPE Type, CString Address, CString Length, CString Data);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_MemoryReadProtectionData								//
// Description    : Read memory card's Protection data. (for SLE4442)				//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
// Output         : Data: Get memory card's Protection data.						//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_MemoryReadProtectionData(LPSHStruct psh, Interface_TYPE Type, CString *Data);

//////////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_MemoryWriteProtectionData									//
// Description    : Write memory card's protection data. (for SLE4442)					//
// Input          : Type: Select interface type.										//
//						 Type_HID: Use for HID device.									//
//						 Type_COM: Use for Com port device.								//
//					Address: Set write memory card's start address.(String: 0000~001F)	//
//					Length: Write data length.(String: 01~FF)							//
//					Data: Write data.(String: 1*2 ~ 255*2 words)						//
// Output         : None																//
// Return         : True or False														//
//////////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_MemoryWriteProtectionData(LPSHStruct psh, Interface_TYPE Type, CString Address, CString Length, CString Data);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_PiccActivate											//
// Description    : Active RF card.													//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
// Output         : Cardno: RF card's number.										//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_PiccActivate(LPSHStruct psh, Interface_TYPE Type, CString *Cardno);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_PiccDeactivate											//
// Description    : Stop RF card operation.											//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
// Output         : None															//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_PiccDeactivate(LPSHStruct psh, Interface_TYPE Type);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_PiccRats												//
// Description    : RF card activated and repond ATS.								//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
// Output         : Ats: Get RF card's ATS(answer to select) values.				//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_PiccRats(LPSHStruct psh, Interface_TYPE Type, CString *Ats);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_PiccAccess												//
// Description    : Send APDU to RF card, and then get response APDU.				//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
//					Capdu: Write APDU command.										//
// Output         : Rapdu: Read response APDU.										//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_PiccAccess(LPSHStruct psh, Interface_TYPE Type, CString Capdu, CString *Rapdu);

//////////////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_MifareAuth														//
// Description    : Mifare card uses the corresponding key for the authentication.			//
// Input          : Type: Select interface type.											//
//						 Type_HID: Use for HID device.										//
//						 Type_COM: Use for Com port device.									//
//					KeyType: Select authentication key type.								//
//							 0x41: Key A													//
//							 0x42: Key B													//
//					Sector: Mifare card's memory location.									//
//							 0x00: Sector 0 ~ 0x0F: Sector 15								//
//					Key: The corresponding key for the authentication.(String: 6*2 words)	//
// Output         : None																	//
// Return         : True or False															//
//////////////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_MifareAuth(LPSHStruct psh, Interface_TYPE Type, INT KeyType, INT Sector, CString Key);

//////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_MifareReadBlock											//
// Description    : Read mifare card's block after a successful authentication.		//
// Input          : Type: Select interface type.									//
//						 Type_HID: Use for HID device.								//
//						 Type_COM: Use for Com port device.							//
//					Block: Mifare card's memory location.(Sector * 4 + Block)		//
//							0x00: Sector 0, Block 0 ~ 0x3F: Sector 15, Block 3		//
// Output         : Data: The contents of a block of mifare card.					//
// Return         : True or False													//
//////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_MifareReadBlock(LPSHStruct psh, Interface_TYPE Type, INT Block, CString *Data);

//////////////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_MifareWriteBlock												//
// Description    : Write mifare card's block after a successful authentication.			//
// Input          : Type: Select interface type.											//
//						 Type_HID: Use for HID device.										//
//						 Type_COM: Use for Com port device.									//
//					Block: Mifare card's memory location.(Sector * 4 + Block)				//
//							0x00: Sector 0, Block 0 ~ 0x3F: Sector 15, Block 3				//
//					Data: The contents of a block of mifare card.(String: 16*2 words)		//
// Output         : None																	//
// Return         : True or False															//
//////////////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_MifareWriteBlock(LPSHStruct psh, Interface_TYPE Type, INT Block, CString Data);

//////////////////////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_MifareIncrement															//
// Description    : Increment value write to mifare card's block after a successful authentication.	//
// Input          : Type: Select interface type.													//
//						 Type_HID: Use for HID device.												//
//						 Type_COM: Use for Com port device.											//
//					Block: Mifare card's memory location.(Sector * 4 + Block)						//
//							0x00: Sector 0, Block 0 ~ 0x3F: Sector 15, Block 3						//
//					Value: The Increment value of mifare card's block.(String: 4*2 words)			//
// Output         : None																			//
// Return         : True or False																	//
//////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_MifareIncrement(LPSHStruct psh, Interface_TYPE Type, INT Block, CString Value);

//////////////////////////////////////////////////////////////////////////////////////////////////////
// Function Name  : SinMPay_MifareDecrement															//
// Description    : Decrement value write to mifare card's block after a successful authentication.	//
// Input          : Type: Select interface type.													//
//						 Type_HID: Use for HID device.												//
//						 Type_COM: Use for Com port device.											//
//					Block: Mifare card's memory location.(Sector * 4 + Block)						//
//							0x00: Sector 0, Block 0 ~ 0x3F: Sector 15, Block 3						//
//					Value: The Decrement value of mifare card's block.(String: 4*2 words)			//
// Output         : None																			//
// Return         : True or False																	//
//////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL SinMPay_MifareDecrement(LPSHStruct psh, Interface_TYPE Type, INT Block, CString Value);

#endif